@@ -5,7 +5,7 @@
 
     The MessageCatalogBuilder class.
 
-    :copyright: Copyright 2007-2016 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2017 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
@@ -18,10 +18,11 @@ from datetime import datetime, tzinfo, timedelta
 from collections import defaultdict
 from uuid import uuid4
 
-from six import iteritems
+from six import iteritems, StringIO
 
 from sphinx.builders import Builder
 from sphinx.util import split_index_msg
+from sphinx.util.tags import Tags
 from sphinx.util.nodes import extract_messages, traverse_translatable_index
 from sphinx.util.osutil import safe_relpath, ensuredir, canon_path
 from sphinx.util.i18n import find_catalog
@@ -79,6 +80,16 @@ class MsgOrigin(object):
         self.uid = uuid4().hex
 
 
+class I18nTags(Tags):
+    """Dummy tags module for I18nBuilder.
+
+    To translate all text inside of only nodes, this class
+    always returns True value even if no tags are defined.
+    """
+    def eval_condition(self, condition):
+        return True
+
+
 class I18nBuilder(Builder):
     """
     General i18n builder.
@@ -93,6 +104,7 @@ class I18nBuilder(Builder):
 
     def init(self):
         Builder.init(self)
+        self.tags = I18nTags()
         self.catalogs = defaultdict(Catalog)
 
     def get_target_uri(self, docname, typ=None):
@@ -154,6 +166,20 @@ class LocalTimeZone(tzinfo):
 ltz = LocalTimeZone()
 
 
+def should_write(filepath, new_content):
+    if not path.exists(filepath):
+        return True
+    with open(filepath, 'r', encoding='utf-8') as oldpot:
+        old_content = oldpot.read()
+        old_header_index = old_content.index('"POT-Creation-Date:')
+        new_header_index = old_content.index('"POT-Creation-Date:')
+        old_body_index = old_content.index('"PO-Revision-Date:')
+        new_body_index = new_content.index('"PO-Revision-Date:')
+        return ((old_content[:old_header_index] != new_content[:new_header_index]) or
+                (new_content[new_body_index:] != old_content[old_body_index:]))
+    return True
+
+
 class MessageCatalogBuilder(I18nBuilder):
     """
     Builds gettext-style message catalogs (.pot files).
@@ -212,29 +238,48 @@ class MessageCatalogBuilder(I18nBuilder):
             ensuredir(path.join(self.outdir, path.dirname(textdomain)))
 
             pofn = path.join(self.outdir, textdomain + '.pot')
-            pofile = open(pofn, 'w', encoding='utf-8')
-            try:
-                pofile.write(POHEADER % data)
 
-                for message in catalog.messages:
-                    positions = catalog.metadata[message]
+            output = StringIO()
+            output.write(POHEADER % data)
 
-                    if self.config.gettext_location:
-                        # generate "#: file1:line1\n#: file2:line2 ..."
-                        pofile.write("#: %s\n" % "\n#: ".join(
-                            "%s:%s" % (canon_path(
-                                safe_relpath(source, self.outdir)), line)
-                            for source, line, _ in positions))
-                    if self.config.gettext_uuid:
-                        # generate "# uuid1\n# uuid2\n ..."
-                        pofile.write("# %s\n" % "\n# ".join(
-                            uid for _, _, uid in positions))
+            for message in catalog.messages:
+                positions = catalog.metadata[message]
 
-                    # message contains *one* line of text ready for translation
-                    message = message.replace('\\', r'\\'). \
-                        replace('"', r'\"'). \
-                        replace('\n', '\\n"\n"')
-                    pofile.write('msgid "%s"\nmsgstr ""\n\n' % message)
+                if self.config.gettext_location:
+                    # generate "#: file1:line1\n#: file2:line2 ..."
+                    output.write("#: %s\n" % "\n#: ".join(
+                        "%s:%s" % (canon_path(
+                            safe_relpath(source, self.outdir)), line)
+                        for source, line, _ in positions))
+                if self.config.gettext_uuid:
+                    # generate "# uuid1\n# uuid2\n ..."
+                    output.write("# %s\n" % "\n# ".join(
+                        uid for _, _, uid in positions))
 
-            finally:
-                pofile.close()
+                # message contains *one* line of text ready for translation
+                message = message.replace('\\', r'\\'). \
+                    replace('"', r'\"'). \
+                    replace('\n', '\\n"\n"')
+                output.write('msgid "%s"\nmsgstr ""\n\n' % message)
+
+            content = output.getvalue()
+
+            if should_write(pofn, content):
+                with open(pofn, 'w', encoding='utf-8') as pofile:
+                    pofile.write(content)
+
+
+def setup(app):
+    app.add_builder(MessageCatalogBuilder)
+
+    app.add_config_value('gettext_compact', True, 'gettext')
+    app.add_config_value('gettext_location', True, 'gettext')
+    app.add_config_value('gettext_uuid', False, 'gettext')
+    app.add_config_value('gettext_auto_build', True, 'env')
+    app.add_config_value('gettext_additional_targets', [], 'env')
+
+    return {
+        'version': 'builtin',
+        'parallel_read_safe': True,
+        'parallel_write_safe': True,
+    }
