@@ -6,19 +6,21 @@
     Build HTML help support files.
     Parts adapted from Python's Doc/tools/prechm.py.
 
-    :copyright: Copyright 2007-2010 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2017 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
+from __future__ import print_function
 
 import os
-import cgi
 import codecs
 from os import path
 
 from docutils import nodes
 
 from sphinx import addnodes
+from sphinx.util.osutil import make_filename
 from sphinx.builders.html import StandaloneHTMLBuilder
+from sphinx.util.pycompat import htmlescape
 
 
 # Project file (*.hhp) template.  'outname' is the file basename (like
@@ -62,7 +64,7 @@ Binary Index=No
 Compiled file=%(outname)s.chm
 Contents file=%(outname)s.hhc
 Default Window=%(outname)s
-Default topic=index.html
+Default topic=%(master_doc)s
 Display compile progress=No
 Full text search stop list file=%(outname)s.stp
 Full-text search=Yes
@@ -72,7 +74,7 @@ Title=%(title)s
 
 [WINDOWS]
 %(outname)s="%(title)s","%(outname)s.hhc","%(outname)s.hhk",\
-"index.html","index.html",,,,,0x63520,220,0x10384e,[0,0,1024,768],,,,,,,0
+"%(master_doc)s","%(master_doc)s",,,,,0x63520,220,0x10384e,[0,0,1024,768],,,,,,,0
 
 [FILES]
 '''
@@ -124,19 +126,32 @@ was  will  with
 # See http://msdn.microsoft.com/en-us/library/ms930130.aspx for more.
 chm_locales = {
     # lang:   LCID,  encoding
-    'cs':    (0x405, 'iso8859_2'),
-    'de':    (0x407, 'iso8859_1'),
-    'en':    (0x409, 'iso8859_1'),
-    'es':    (0x40a, 'iso8859_1'),
-    'fi':    (0x40b, 'iso8859_1'),
-    'fr':    (0x40c, 'iso8859_1'),
-    'it':    (0x410, 'iso8859_1'),
+    'ca':    (0x403, 'cp1252'),
+    'cs':    (0x405, 'cp1250'),
+    'da':    (0x406, 'cp1252'),
+    'de':    (0x407, 'cp1252'),
+    'en':    (0x409, 'cp1252'),
+    'es':    (0x40a, 'cp1252'),
+    'et':    (0x425, 'cp1257'),
+    'fa':    (0x429, 'cp1256'),
+    'fi':    (0x40b, 'cp1252'),
+    'fr':    (0x40c, 'cp1252'),
+    'hr':    (0x41a, 'cp1250'),
+    'hu':    (0x40e, 'cp1250'),
+    'it':    (0x410, 'cp1252'),
     'ja':    (0x411, 'cp932'),
-    'nl':    (0x413, 'iso8859_1'),
-    'pl':    (0x415, 'iso8859_2'),
-    'pt_BR': (0x416, 'iso8859_1'),
+    'ko':    (0x412, 'cp949'),
+    'lt':    (0x427, 'cp1257'),
+    'lv':    (0x426, 'cp1257'),
+    'nl':    (0x413, 'cp1252'),
+    'no_NB': (0x414, 'cp1252'),
+    'pl':    (0x415, 'cp1250'),
+    'pt_BR': (0x416, 'cp1252'),
     'ru':    (0x419, 'cp1251'),
-    'sl':    (0x424, 'iso8859_2'),
+    'sk':    (0x41b, 'cp1250'),
+    'sl':    (0x424, 'cp1250'),
+    'sv':    (0x41d, 'cp1252'),
+    'tr':    (0x41f, 'cp1254'),
     'uk_UA': (0x422, 'cp1251'),
     'zh_CN': (0x804, 'cp936'),
     'zh_TW': (0x404, 'cp950'),
@@ -159,13 +174,17 @@ class HTMLHelpBuilder(StandaloneHTMLBuilder):
     # don't add sidebar etc.
     embedded = True
 
+    # don't generate search index or include search page
+    search = False
+
     lcid = 0x409
-    encoding = 'iso8859_1'
+    encoding = 'cp1252'
 
     def init(self):
         StandaloneHTMLBuilder.init(self)
         # the output files for HTML help must be .html only
         self.out_suffix = '.html'
+        self.link_suffix = '.html'
         # determine the correct locale setting
         locale = chm_locales.get(self.config.language)
         if locale is not None:
@@ -174,54 +193,59 @@ class HTMLHelpBuilder(StandaloneHTMLBuilder):
     def open_file(self, outdir, basename, mode='w'):
         # open a file with the correct encoding for the selected language
         return codecs.open(path.join(outdir, basename), mode,
-            self.encoding, 'xmlcharrefreplace')
+                           self.encoding, 'xmlcharrefreplace')
 
     def handle_finish(self):
         self.build_hhx(self.outdir, self.config.htmlhelp_basename)
 
+    def write_doc(self, docname, doctree):
+        for node in doctree.traverse(nodes.reference):
+            # add ``target=_blank`` attributes to external links
+            if node.get('internal') is None and 'refuri' in node:
+                node['target'] = '_blank'
+
+        StandaloneHTMLBuilder.write_doc(self, docname, doctree)
+
     def build_hhx(self, outdir, outname):
         self.info('dumping stopword list...')
-        f = self.open_file(outdir, outname+'.stp')
-        try:
+        with self.open_file(outdir, outname + '.stp') as f:
             for word in sorted(stopwords):
-                print >>f, word
-        finally:
-            f.close()
+                print(word, file=f)
 
         self.info('writing project file...')
-        f = self.open_file(outdir, outname+'.hhp')
-        try:
-            f.write(project_template % {'outname': outname,
-                                        'title': self.config.html_title,
-                                        'version': self.config.version,
-                                        'project': self.config.project,
-                                        'lcid': self.lcid})
+        with self.open_file(outdir, outname + '.hhp') as f:
+            f.write(project_template % {
+                'outname': outname,
+                'title': self.config.html_title,
+                'version': self.config.version,
+                'project': self.config.project,
+                'lcid': self.lcid,
+                'master_doc': self.config.master_doc + self.out_suffix
+            })
             if not outdir.endswith(os.sep):
                 outdir += os.sep
             olen = len(outdir)
             for root, dirs, files in os.walk(outdir):
-                staticdir = (root == path.join(outdir, '_static'))
+                staticdir = root.startswith(path.join(outdir, '_static'))
                 for fn in files:
                     if (staticdir and not fn.endswith('.js')) or \
-                           fn.endswith('.html'):
-                        print >>f, path.join(root, fn)[olen:].replace(os.sep,
-                                                                      '\\')
-        finally:
-            f.close()
+                       fn.endswith('.html'):
+                        print(path.join(root, fn)[olen:].replace(os.sep, '\\'),
+                              file=f)
 
         self.info('writing TOC file...')
-        f = self.open_file(outdir, outname+'.hhc')
-        try:
+        with self.open_file(outdir, outname + '.hhc') as f:
             f.write(contents_header)
             # special books
             f.write('<LI> ' + object_sitemap % (self.config.html_short_title,
-                                                'index.html'))
+                                                self.config.master_doc + self.out_suffix))
             for indexname, indexcls, content, collapse in self.domain_indices:
                 f.write('<LI> ' + object_sitemap % (indexcls.localname,
                                                     '%s.html' % indexname))
             # the TOC
             tocdoc = self.env.get_and_resolve_doctree(
                 self.config.master_doc, self, prune_toctrees=False)
+
             def write_toc(node, ullevel=0):
                 if isinstance(node, nodes.list_item):
                     f.write('<LI> ')
@@ -229,48 +253,48 @@ class HTMLHelpBuilder(StandaloneHTMLBuilder):
                         write_toc(subnode, ullevel)
                 elif isinstance(node, nodes.reference):
                     link = node['refuri']
-                    title = cgi.escape(node.astext()).replace('"','&quot;')
+                    title = htmlescape(node.astext()).replace('"', '&quot;')
                     f.write(object_sitemap % (title, link))
                 elif isinstance(node, nodes.bullet_list):
                     if ullevel != 0:
                         f.write('<UL>\n')
                     for subnode in node:
-                        write_toc(subnode, ullevel+1)
+                        write_toc(subnode, ullevel + 1)
                     if ullevel != 0:
                         f.write('</UL>\n')
                 elif isinstance(node, addnodes.compact_paragraph):
                     for subnode in node:
                         write_toc(subnode, ullevel)
+
             def istoctree(node):
                 return isinstance(node, addnodes.compact_paragraph) and \
-                       node.has_key('toctree')
+                    'toctree' in node
             for node in tocdoc.traverse(istoctree):
                 write_toc(node)
             f.write(contents_footer)
-        finally:
-            f.close()
 
         self.info('writing index file...')
         index = self.env.create_index(self)
-        f = self.open_file(outdir, outname+'.hhk')
-        try:
+        with self.open_file(outdir, outname + '.hhk') as f:
             f.write('<UL>\n')
+
             def write_index(title, refs, subitems):
                 def write_param(name, value):
-                    item = '    <param name="%s" value="%s">\n' % (name, value)
-                    f.write(item.encode('ascii', 'xmlcharrefreplace'))
-                title = cgi.escape(title)
+                    item = '    <param name="%s" value="%s">\n' % \
+                        (name, value)
+                    f.write(item)
+                title = htmlescape(title)
                 f.write('<LI> <OBJECT type="text/sitemap">\n')
                 write_param('Keyword', title)
                 if len(refs) == 0:
                     write_param('See Also', title)
                 elif len(refs) == 1:
-                    write_param('Local', refs[0])
+                    write_param('Local', refs[0][1])
                 else:
                     for i, ref in enumerate(refs):
                         # XXX: better title?
-                        write_param('Name', '[%d] %s' % (i, ref))
-                        write_param('Local', ref)
+                        write_param('Name', '[%d] %s' % (i, ref[1]))
+                        write_param('Local', ref[1])
                 f.write('</OBJECT>\n')
                 if subitems:
                     f.write('<UL> ')
@@ -278,8 +302,19 @@ class HTMLHelpBuilder(StandaloneHTMLBuilder):
                         write_index(subitem[0], subitem[1], [])
                     f.write('</UL>')
             for (key, group) in index:
-                for title, (refs, subitems) in group:
+                for title, (refs, subitems, key_) in group:
                     write_index(title, refs, subitems)
             f.write('</UL>\n')
-        finally:
-            f.close()
+
+
+def setup(app):
+    app.setup_extension('sphinx.builders.html')
+    app.add_builder(HTMLHelpBuilder)
+
+    app.add_config_value('htmlhelp_basename', lambda self: make_filename(self.project), None)
+
+    return {
+        'version': 'builtin',
+        'parallel_read_safe': True,
+        'parallel_write_safe': True,
+    }
