@@ -5,7 +5,7 @@
 
     Plain-text Sphinx builder.
 
-    :copyright: Copyright 2007-2010 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2017 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
@@ -14,8 +14,8 @@ from os import path
 
 from docutils.io import StringOutput
 
-from sphinx.util import ensuredir, os_path
 from sphinx.builders import Builder
+from sphinx.util.osutil import ensuredir, os_path
 from sphinx.writers.text import TextWriter
 
 
@@ -23,6 +23,7 @@ class TextBuilder(Builder):
     name = 'text'
     format = 'text'
     out_suffix = '.txt'
+    allow_parallel = True
 
     def init(self):
         pass
@@ -53,18 +54,29 @@ class TextBuilder(Builder):
         self.writer = TextWriter(self)
 
     def write_doc(self, docname, doctree):
+        self.current_docname = docname
         destination = StringOutput(encoding='utf-8')
         self.writer.write(doctree, destination)
         outfilename = path.join(self.outdir, os_path(docname) + self.out_suffix)
         ensuredir(path.dirname(outfilename))
         try:
-            f = codecs.open(outfilename, 'w', 'utf-8')
-            try:
+            with codecs.open(outfilename, 'w', 'utf-8') as f:
                 f.write(self.writer.output)
-            finally:
-                f.close()
-        except (IOError, OSError), err:
+        except (IOError, OSError) as err:
             self.warn("error writing file %s: %s" % (outfilename, err))
 
     def finish(self):
         pass
+
+
+def setup(app):
+    app.add_builder(TextBuilder)
+
+    app.add_config_value('text_sectionchars', '*=-~"+`', 'env')
+    app.add_config_value('text_newlines', 'unix', 'env')
+
+    return {
+        'version': 'builtin',
+        'parallel_read_safe': True,
+        'parallel_write_safe': True,
+    }
