@@ -5,7 +5,7 @@
 
     The JavaScript domain.
 
-    :copyright: Copyright 2007-2010 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2017 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
@@ -13,8 +13,8 @@ from sphinx import addnodes
 from sphinx.domains import Domain, ObjType
 from sphinx.locale import l_, _
 from sphinx.directives import ObjectDescription
-from sphinx.domains.python import py_paramlist_re as js_paramlist_re
 from sphinx.roles import XRefRole
+from sphinx.domains.python import _pseudo_parse_arglist
 from sphinx.util.nodes import make_refnode
 from sphinx.util.docfields import Field, GroupedField, TypedField
 
@@ -34,6 +34,7 @@ class JSObject(ObjectDescription):
         sig = sig.strip()
         if '(' in sig and sig[-1:] == ')':
             prefix, arglist = sig.split('(', 1)
+            prefix = prefix.strip()
             arglist = arglist[:-1].strip()
         else:
             prefix = sig
@@ -44,7 +45,7 @@ class JSObject(ObjectDescription):
             nameprefix = None
             name = prefix
 
-        objectname = self.env.temp_data.get('js:object')
+        objectname = self.env.ref_context.get('js:object')
         if nameprefix:
             if objectname:
                 # someone documenting the method of an attribute of the current
@@ -68,33 +69,15 @@ class JSObject(ObjectDescription):
             signode += addnodes.desc_addname(nameprefix + '.', nameprefix + '.')
         signode += addnodes.desc_name(name, name)
         if self.has_arguments:
-            signode += addnodes.desc_parameterlist()
-        if not arglist:
-            return fullname, nameprefix
-
-        stack = [signode[-1]]
-        for token in js_paramlist_re.split(arglist):
-            if token == '[':
-                opt = addnodes.desc_optional()
-                stack[-1] += opt
-                stack.append(opt)
-            elif token == ']':
-                try:
-                    stack.pop()
-                except IndexError:
-                    raise ValueError()
-            elif not token or token == ',' or token.isspace():
-                pass
+            if not arglist:
+                signode += addnodes.desc_parameterlist()
             else:
-                token = token.strip()
-                stack[-1] += addnodes.desc_parameter(token, token)
-        if len(stack) != 1:
-            raise ValueError()
+                _pseudo_parse_arglist(signode, arglist)
         return fullname, nameprefix
 
     def add_target_and_index(self, name_obj, sig, signode):
         objectname = self.options.get(
-            'object', self.env.temp_data.get('js:object'))
+            'object', self.env.ref_context.get('js:object'))
         fullname = name_obj[0]
         if fullname not in self.state.document.ids:
             signode['names'].append(fullname)
@@ -103,18 +86,18 @@ class JSObject(ObjectDescription):
             self.state.document.note_explicit_target(signode)
             objects = self.env.domaindata['js']['objects']
             if fullname in objects:
-                self.env.warn(
-                    self.env.docname,
+                self.state_machine.reporter.warning(
                     'duplicate object description of %s, ' % fullname +
                     'other instance in ' +
                     self.env.doc2path(objects[fullname][0]),
-                    self.lineno)
+                    line=self.lineno)
             objects[fullname] = self.env.docname, self.objtype
 
         indextext = self.get_index_text(objectname, name_obj)
         if indextext:
             self.indexnode['entries'].append(('single', indextext,
-                                              fullname, fullname))
+                                              fullname.replace('$', '_S_'),
+                                              '', None))
 
     def get_index_text(self, objectname, name_obj):
         name, obj = name_obj
@@ -144,18 +127,20 @@ class JSCallable(JSObject):
                      can_collapse=True),
         Field('returnvalue', label=l_('Returns'), has_arg=False,
               names=('returns', 'return')),
+        Field('returntype', label=l_('Return type'), has_arg=False,
+              names=('rtype',)),
     ]
 
 
 class JSConstructor(JSCallable):
-    """Like a callable but with a different prefix"""
+    """Like a callable but with a different prefix."""
     display_prefix = 'class '
 
 
 class JSXRefRole(XRefRole):
     def process_link(self, env, refnode, has_explicit_title, title, target):
         # basically what sphinx.domains.python.PyXRefRole does
-        refnode['js:object'] = env.temp_data.get('js:object')
+        refnode['js:object'] = env.ref_context.get('js:object')
         if not has_explicit_title:
             title = title.lstrip('.')
             target = target.lstrip('~')
@@ -163,7 +148,7 @@ class JSXRefRole(XRefRole):
                 title = title[1:]
                 dot = title.rfind('.')
                 if dot != -1:
-                    title = title[dot+1:]
+                    title = title[dot + 1:]
         if target[0:1] == '.':
             target = target[1:]
             refnode['refspecific'] = True
@@ -194,14 +179,20 @@ class JavaScriptDomain(Domain):
         'attr':  JSXRefRole(),
     }
     initial_data = {
-        'objects': {}, # fullname -> docname, objtype
+        'objects': {},  # fullname -> docname, objtype
     }
 
     def clear_doc(self, docname):
-        for fullname, (fn, _) in self.data['objects'].items():
+        for fullname, (fn, _l) in list(self.data['objects'].items()):
             if fn == docname:
                 del self.data['objects'][fullname]
 
+    def merge_domaindata(self, docnames, otherdata):
+        # XXX check duplicates
+        for fullname, (fn, objtype) in otherdata['objects'].items():
+            if fn in docnames:
+                self.data['objects'][fullname] = (fn, objtype)
+
     def find_obj(self, env, obj, name, typ, searchorder=0):
         if name[-2:] == '()':
             name = name[:-2]
@@ -226,8 +217,30 @@ class JavaScriptDomain(Domain):
         name, obj = self.find_obj(env, objectname, target, typ, searchorder)
         if not obj:
             return None
-        return make_refnode(builder, fromdocname, obj[0], name, contnode, name)
+        return make_refnode(builder, fromdocname, obj[0],
+                            name.replace('$', '_S_'), contnode, name)
+
+    def resolve_any_xref(self, env, fromdocname, builder, target, node,
+                         contnode):
+        objectname = node.get('js:object')
+        name, obj = self.find_obj(env, objectname, target, None, 1)
+        if not obj:
+            return []
+        return [('js:' + self.role_for_objtype(obj[1]),
+                 make_refnode(builder, fromdocname, obj[0],
+                              name.replace('$', '_S_'), contnode, name))]
 
     def get_objects(self):
-        for refname, (docname, type) in self.data['objects'].iteritems():
-            yield refname, refname, type, docname, refname, 1
+        for refname, (docname, type) in list(self.data['objects'].items()):
+            yield refname, refname, type, docname, \
+                refname.replace('$', '_S_'), 1
+
+
+def setup(app):
+    app.add_domain(JavaScriptDomain)
+
+    return {
+        'version': 'builtin',
+        'parallel_read_safe': True,
+        'parallel_write_safe': True,
+    }
