@@ -6,10 +6,11 @@
     Contains SphinxError and a few subclasses (in an extra module to avoid
     circular import problems).
 
-    :copyright: Copyright 2007-2009 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2017 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
+
 class SphinxError(Exception):
     """
     Base class for Sphinx errors that are shown to the user in a nicer
@@ -44,5 +45,33 @@ class ExtensionError(SphinxError):
         return parent_str
 
 
+class ConfigError(SphinxError):
+    category = 'Configuration error'
+
+
 class ThemeError(SphinxError):
     category = 'Theme error'
+
+
+class VersionRequirementError(SphinxError):
+    category = 'Sphinx version error'
+
+
+class PycodeError(Exception):
+    def __str__(self):
+        res = self.args[0]
+        if len(self.args) > 1:
+            res += ' (exception was: %r)' % self.args[1]
+        return res
+
+
+class SphinxParallelError(SphinxError):
+
+    category = 'Sphinx parallel build error'
+
+    def __init__(self, message, traceback):
+        self.message = message
+        self.traceback = traceback
+
+    def __str__(self):
+        return self.message
