@@ -16,16 +16,19 @@
     namespace of the project configuration (that is, all variables from
     ``conf.py`` are available.)
 
-    :copyright: Copyright 2007-2010 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2017 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
 from docutils import nodes
 
+import sphinx
+from sphinx.util.nodes import set_source_info
 from sphinx.util.compat import Directive
 
 
-class ifconfig(nodes.Element): pass
+class ifconfig(nodes.Element):
+    pass
 
 
 class IfConfig(Directive):
@@ -39,7 +42,7 @@ class IfConfig(Directive):
     def run(self):
         node = ifconfig()
         node.document = self.state.document
-        node.line = self.lineno
+        set_source_info(self, node)
         node['expr'] = self.arguments[0]
         self.state.nested_parse(self.content, self.content_offset,
                                 node, match_titles=1)
@@ -47,12 +50,13 @@ class IfConfig(Directive):
 
 
 def process_ifconfig_nodes(app, doctree, docname):
-    ns = app.config.__dict__.copy()
+    ns = dict((k, app.config[k]) for k in app.config.values)
+    ns.update(app.config.__dict__.copy())
     ns['builder'] = app.builder.name
     for node in doctree.traverse(ifconfig):
         try:
             res = eval(node['expr'], ns)
-        except Exception, err:
+        except Exception as err:
             # handle exceptions in a clean fashion
             from traceback import format_exception_only
             msg = ''.join(format_exception_only(err.__class__, err))
@@ -71,3 +75,4 @@ def setup(app):
     app.add_node(ifconfig)
     app.add_directive('ifconfig', IfConfig)
     app.connect('doctree-resolved', process_ifconfig_nodes)
+    return {'version': sphinx.__display_version__, 'parallel_read_safe': True}
