@@ -5,40 +5,85 @@
 
     Glue code for the jinja2 templating engine.
 
-    :copyright: Copyright 2007-2010 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2017 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
 from os import path
 from pprint import pformat
 
+from six import string_types
 from jinja2 import FileSystemLoader, BaseLoader, TemplateNotFound, \
-     contextfunction
+    contextfunction
 from jinja2.utils import open_if_exists
 from jinja2.sandbox import SandboxedEnvironment
 
-from sphinx.util import mtimes_of_files
 from sphinx.application import TemplateBridge
+from sphinx.util.osutil import mtimes_of_files
 
 
 def _tobool(val):
-    if isinstance(val, basestring):
+    if isinstance(val, string_types):
         return val.lower() in ('true', '1', 'yes', 'on')
     return bool(val)
 
+
+def _toint(val):
+    try:
+        return int(val)
+    except ValueError:
+        return 0
+
+
+def _slice_index(values, slices):
+    seq = list(values)
+    length = 0
+    for value in values:
+        length += 1 + len(value[1][1])  # count includes subitems
+    items_per_slice = length // slices
+    offset = 0
+    for slice_number in range(slices):
+        count = 0
+        start = offset
+        if slices == slice_number + 1:  # last column
+            offset = len(seq)
+        else:
+            for value in values[offset:]:
+                count += 1 + len(value[1][1])
+                offset += 1
+                if count >= items_per_slice:
+                    break
+        yield seq[start:offset]
+
+
 def accesskey(context, key):
     """Helper to output each access key only once."""
     if '_accesskeys' not in context:
         context.vars['_accesskeys'] = {}
-    if key not in context.vars['_accesskeys']:
+    if key and key not in context.vars['_accesskeys']:
         context.vars['_accesskeys'][key] = 1
         return 'accesskey="%s"' % key
     return ''
 
 
+class idgen(object):
+    def __init__(self):
+        self.id = 0
+
+    def current(self):
+        return self.id
+
+    def __next__(self):
+        self.id += 1
+        return self.id
+    next = __next__  # Python 2/Jinja compatibility
+
+
 class SphinxFileSystemLoader(FileSystemLoader):
-    """FileSystemLoader subclass that is not so strict about '..'
-    entries in template names."""
+    """
+    FileSystemLoader subclass that is not so strict about '..'  entries in
+    template names.
+    """
 
     def get_source(self, environment, template):
         for searchpath in self.searchpath:
@@ -46,12 +91,11 @@ class SphinxFileSystemLoader(FileSystemLoader):
             f = open_if_exists(filename)
             if f is None:
                 continue
-            try:
+            with f:
                 contents = f.read().decode(self.encoding)
-            finally:
-                f.close()
 
             mtime = path.getmtime(filename)
+
             def uptodate():
                 try:
                     return path.getmtime(filename) == mtime
@@ -61,7 +105,6 @@ class SphinxFileSystemLoader(FileSystemLoader):
         raise TemplateNotFound(template)
 
 
-
 class BuiltinTemplateLoader(TemplateBridge, BaseLoader):
     """
     Interfaces the rendering environment of jinja2 for use in Sphinx.
@@ -73,35 +116,43 @@ class BuiltinTemplateLoader(TemplateBridge, BaseLoader):
         # create a chain of paths to search
         if theme:
             # the theme's own dir and its bases' dirs
-            chain = theme.get_dirchain()
+            pathchain = theme.get_dirchain()
             # then the theme parent paths
-            chain.extend(theme.themepath)
+            loaderchain = pathchain + theme.themepath
         elif dirs:
-            chain = list(dirs)
+            pathchain = list(dirs)
+            loaderchain = list(dirs)
         else:
-            chain = []
+            pathchain = []
+            loaderchain = []
 
         # prepend explicit template paths
         self.templatepathlen = len(builder.config.templates_path)
         if builder.config.templates_path:
-            chain[0:0] = [path.join(builder.confdir, tp)
-                          for tp in builder.config.templates_path]
+            cfg_templates_path = [path.join(builder.confdir, tp)
+                                  for tp in builder.config.templates_path]
+            pathchain[0:0] = cfg_templates_path
+            loaderchain[0:0] = cfg_templates_path
 
         # store it for use in newest_template_mtime
-        self.pathchain = chain
+        self.pathchain = pathchain
 
         # make the paths into loaders
-        self.loaders = map(SphinxFileSystemLoader, chain)
+        self.loaders = [SphinxFileSystemLoader(x) for x in loaderchain]
 
-        use_i18n = builder.translator is not None
+        use_i18n = builder.app.translator is not None
         extensions = use_i18n and ['jinja2.ext.i18n'] or []
         self.environment = SandboxedEnvironment(loader=self,
                                                 extensions=extensions)
         self.environment.filters['tobool'] = _tobool
+        self.environment.filters['toint'] = _toint
+        self.environment.filters['slice_index'] = _slice_index
         self.environment.globals['debug'] = contextfunction(pformat)
         self.environment.globals['accesskey'] = contextfunction(accesskey)
+        self.environment.globals['idgen'] = idgen
         if use_i18n:
-            self.environment.install_gettext_translations(builder.translator)
+            self.environment.install_gettext_translations(
+                builder.app.translator)
 
     def render(self, template, context):
         return self.environment.get_template(template).render(context)
