@@ -5,7 +5,7 @@
 
     Theming support for HTML builders.
 
-    :copyright: Copyright 2007-2016 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2017 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
@@ -68,14 +68,19 @@ class Theme(object):
 
     @classmethod
     def load_extra_theme(cls, name):
-        if name in ('alabaster', 'sphinx_rtd_theme'):
+        themes = ['alabaster']
+        try:
+            import sphinx_rtd_theme
+            themes.append('sphinx_rtd_theme')
+        except ImportError:
+            pass
+        if name in themes:
             if name == 'alabaster':
                 import alabaster
                 themedir = alabaster.get_path()
                 # alabaster theme also requires 'alabaster' extension, it will be loaded
                 # at sphinx.application module.
             elif name == 'sphinx_rtd_theme':
-                import sphinx_rtd_theme
                 themedir = sphinx_rtd_theme.get_html_theme_path()
             else:
                 raise NotImplementedError('Programming Error')
@@ -96,8 +101,13 @@ class Theme(object):
         if name not in self.themes:
             self.load_extra_theme(name)
             if name not in self.themes:
-                raise ThemeError('no theme named %r found '
-                                 '(missing theme.conf?)' % name)
+                if name == 'sphinx_rtd_theme':
+                    raise ThemeError('sphinx_rtd_theme is no longer a hard dependency '
+                                     'since version 1.4.0. Please install it manually.'
+                                     '(pip install sphinx_rtd_theme)')
+                else:
+                    raise ThemeError('no theme named %r found '
+                                     '(missing theme.conf?)' % name)
         self.name = name
 
         # Do not warn yet -- to be compatible with old Sphinxes, people *have*
@@ -123,9 +133,8 @@ class Theme(object):
                 dirname = path.dirname(name)
                 if not path.isdir(path.join(self.themedir, dirname)):
                     os.makedirs(path.join(self.themedir, dirname))
-                fp = open(path.join(self.themedir, name), 'wb')
-                fp.write(tinfo.read(name))
-                fp.close()
+                with open(path.join(self.themedir, name), 'wb') as fp:
+                    fp.write(tinfo.read(name))
 
         self.themeconf = configparser.RawConfigParser()
         self.themeconf.read(path.join(self.themedir, THEMECONF))
@@ -216,7 +225,7 @@ def load_theme_plugins():
         func_or_path = plugin.load()
         try:
             path = func_or_path()
-        except:
+        except Exception:
             path = func_or_path
 
         if isinstance(path, string_types):
