@@ -5,7 +5,7 @@
 
     Helpers for inspecting Python modules.
 
-    :copyright: Copyright 2007-2016 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2017 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
@@ -20,7 +20,7 @@ from sphinx.util import force_decode
 # relatively import this module
 inspect = __import__('inspect')
 
-memory_address_re = re.compile(r' at 0x[0-9a-f]{8,16}(?=>)')
+memory_address_re = re.compile(r' at 0x[0-9a-f]{8,16}(?=>)', re.IGNORECASE)
 
 
 if PY3:
@@ -60,7 +60,7 @@ if PY3:
             raise TypeError('%r is not a Python function' % func)
         return inspect.getfullargspec(func)
 
-else:  # 2.6, 2.7
+else:  # 2.7
     from functools import partial
 
     def getargspec(func):
@@ -94,6 +94,25 @@ else:  # 2.6, 2.7
                     pass
         return inspect.ArgSpec(args, varargs, varkw, func_defaults)
 
+try:
+    import enum
+except ImportError:
+    enum = None
+
+
+def isenumclass(x):
+    """Check if the object is subclass of enum."""
+    if enum is None:
+        return False
+    return inspect.isclass(x) and issubclass(x, enum.Enum)
+
+
+def isenumattribute(x):
+    """Check if the object is attribute of enum."""
+    if enum is None:
+        return False
+    return isinstance(x, enum.Enum)
+
 
 def isdescriptor(x):
     """Check if the object is some kind of descriptor."""
