@@ -5,7 +5,7 @@
 
     Simple requests package loader
 
-    :copyright: Copyright 2007-2016 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2017 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
@@ -36,30 +36,44 @@ except ImportError:
         # for requests < 2.4.0
         InsecureRequestWarning = None
 
-# try to load requests[security]
 try:
-    pkg_resources.require(['requests[security]'])
-except (pkg_resources.DistributionNotFound,
-        pkg_resources.VersionConflict):
+    from requests.packages.urllib3.exceptions import InsecurePlatformWarning
+except ImportError:
+    try:
+        # for Debian-jessie
+        from urllib3.exceptions import InsecurePlatformWarning
+    except ImportError:
+        # for requests < 2.4.0
+        InsecurePlatformWarning = None
+
+# try to load requests[security] (but only if SSL is available)
+try:
     import ssl
-    if not getattr(ssl, 'HAS_SNI', False):
-        # don't complain on each url processed about the SSL issue
-        requests.packages.urllib3.disable_warnings(
-            requests.packages.urllib3.exceptions.InsecurePlatformWarning)
+except ImportError:
+    pass
+else:
+    try:
+        pkg_resources.require(['requests[security]'])
+    except (pkg_resources.DistributionNotFound,
+            pkg_resources.VersionConflict):
+        if not getattr(ssl, 'HAS_SNI', False):
+            # don't complain on each url processed about the SSL issue
+            if InsecurePlatformWarning:
+                requests.packages.urllib3.disable_warnings(InsecurePlatformWarning)
+            warnings.warn(
+                'Some links may return broken results due to being unable to '
+                'check the Server Name Indication (SNI) in the returned SSL cert '
+                'against the hostname in the url requested. Recommended to '
+                'install "requests[security]" as a dependency or upgrade to '
+                'a python version with SNI support (Python 3 and Python 2.7.9+).'
+            )
+    except pkg_resources.UnknownExtra:
         warnings.warn(
             'Some links may return broken results due to being unable to '
             'check the Server Name Indication (SNI) in the returned SSL cert '
             'against the hostname in the url requested. Recommended to '
-            'install "requests[security]" as a dependency or upgrade to '
-            'a python version with SNI support (Python 3 and Python 2.7.9+).'
+            'install requests-2.4.1+.'
         )
-except pkg_resources.UnknownExtra:
-    warnings.warn(
-        'Some links may return broken results due to being unable to '
-        'check the Server Name Indication (SNI) in the returned SSL cert '
-        'against the hostname in the url requested. Recommended to '
-        'install requests-2.4.1+.'
-    )
 
 useragent_header = [('User-Agent',
                      'Mozilla/5.0 (X11; Linux x86_64; rv:25.0) Gecko/20100101 Firefox/25.0')]
