@@ -7,13 +7,13 @@
     test the HTML itself; that's already handled by
     :file:`test_build_html.py`.
 
-    :copyright: Copyright 2007-2016 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2017 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
 import plistlib
 
-from util import with_app
+import pytest
 from path import path
 
 # Use plistlib.load in 3.4 and above
@@ -43,9 +43,10 @@ def check_localization(outdir):
     assert (lprojdir / 'localized.txt').isfile()
 
 
-@with_app(buildername='applehelp', testroot='basic', srcdir='applehelp_output',
-          confoverrides={'applehelp_bundle_id': 'org.sphinx-doc.Sphinx.help',
-                         'applehelp_disable_external_tools': True})
+@pytest.mark.sphinx(
+    'applehelp', testroot='basic', srcdir='applehelp_output',
+    confoverrides={'applehelp_bundle_id': 'org.sphinx-doc.Sphinx.help',
+                   'applehelp_disable_external_tools': True})
 def test_applehelp_output(app, status, warning):
     (app.srcdir / 'en.lproj').makedirs()
     (app.srcdir / 'en.lproj' / 'localized.txt').write_text('')
