@@ -5,7 +5,7 @@
 
     Test the HTML builder and check output against XPath.
 
-    :copyright: Copyright 2007-2016 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2017 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
@@ -17,7 +17,7 @@ from six import PY3
 
 from sphinx import __display_version__
 from util import remove_unicode_literals, strip_escseq
-from etree13 import ElementTree
+import xml.etree.cElementTree as ElementTree
 from html5lib import getTreeBuilder, HTMLParser
 import pytest
 
