@@ -5,16 +5,16 @@
 
     Test sphinx.ext.autosectionlabel extension.
 
-    :copyright: Copyright 2007-2016 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2017 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
 import re
 
-from util import with_app
+import pytest
 
 
-@with_app('html', testroot='ext-autosectionlabel')
+@pytest.mark.sphinx('html', testroot='ext-autosectionlabel')
 def test_autosectionlabel_html(app, status, warning):
     app.builder.build_all()
 
