@@ -5,14 +5,14 @@
 
     Test templating.
 
-    :copyright: Copyright 2007-2016 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2017 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
-from util import with_app
+import pytest
 
 
-@with_app('html', testroot='templating')
+@pytest.mark.sphinx('html', testroot='templating')
 def test_layout_overloading(app, status, warning):
     app.builder.build_update()
 
@@ -21,7 +21,7 @@ def test_layout_overloading(app, status, warning):
     assert '<!-- layout overloading -->' in result
 
 
-@with_app('html', testroot='templating')
+@pytest.mark.sphinx('html', testroot='templating')
 def test_autosummary_class_template_overloading(app, status, warning):
     app.builder.build_update()
 
